--Author PaKettle@Ymail.com 
-- March 2017  
  
PR.Data = {}                 -- to be spickled
PR.Data.MaxId = 0
PR.Data.Usable = 0
PR.Data.RoidStatus = {}
PR.Data.RoidList = {}
PR.Data.Unscanned = 0 
PR.Data.Bots = 0

PR.LoadedSector = 0
PR.ScanCompletion = 0
PR.DisplayColorFlipper = "odd"

PR.SaveSectorTimer = Timer()
PR.SaveSectorSave = 0


function PR.GetRoid(SectorId,RoidId)
   local RoidTable = {["Src"] = "L",["XPV"] = 0 }
   if PR.LoadedSector ~= SectorId then PR.LoadSector(SectorId)  end
   RoidTable = PR.Data.RoidList[RoidId] 
   if RoidTable == nil then RoidTable = {} end   
   RoidTable["Sid"] = SectorId    -- restore sector and roid id tags
   RoidTable["Id"] = RoidId 
   
   return RoidTable
end


function PR.SetRoid(RoidTable)               
   local sectorid = RoidTable["Sid"]
   local roidid = RoidTable["Id"]
   local Color = COLOR.HEXWHITE   
   local RRT = PR.Data.RoidList[roidid] or {}
   
   for Cat,Val in pairs (RoidTable) do
      if Cat ~= 0 then
         if Val ~= nil and Val ~= 0 then RRT[Cat] = Val  end
      end
   end

   PR.LoadSector(sectorid)
   PR.Data.RoidList[roidid] = RRT

   if PR.GetRoidStatus(roidid) == "u" and PR.Config.Quiet == "NO" then
      if PR.Config.COLOR ~= "WHITE" then 
         if  PR.DisplayColorFlipper == "odd" then 
            Color = COLOR.HEXRED
            PR.DisplayColorFlipper = "even"
         else
            PR.DisplayColorFlipper = "odd"
            Color = COLOR.HEXBLUE
         end
      end
      PR.Print ("\t\127"..Color .. "Recording new asteroid Data" .. "\127o")
   end
   PR.SaveSector(sectorid)   
end

function PR.DeleteRoid(sectorid ,roidid) 
   if roidid == nil or sectorid == nil then return end              
   local ldata = {}
   local fname = PR.SystemPrefix + sectorid
   local tempD = unspickle(LoadSystemNotes(fname ))
   
   for Cat,val in pairs (tempD) do
      ldata[Cat] = val           
   end
 
   local code = ldata.RoidStatus[roidid]
   if code == "r" then 
      ldata.RoidList[roidid] = nil
   end
   ldata.RoidStatus[roidid] = "u"
   SaveSystemNotes(spickle(ldata),fname) 
   print("Deleting Asteroid Data")
end


function PR.SaveSector(SectorId)
   if PR.SaveSectorSave == 0 then PR.SaveSectorSave = SectorId end
   if not PR.SaveSectorTimer:IsActive() then
      PR.SaveSectorTimer:SetTimeout(2000000,PR.SaveSectorNow)
   end
end

function PR.SaveSectorNow()
   if PR.LeviDelayCount > 0 then 
      PR.SaveSector(PR.LoadedSector) 
   end    -- dont save anything unless levi scanner done
   if PR.Status ~= "READY" or PR.Levi == 1 then return end
   if PR.SaveSectorTimer:IsActive() then PR.SaveSectorTimer:Kill() end
   local RoidNotes = spickle(PR.Data)
   if PR.SaveSectorSave == PR.LoadedSector then
      SaveSystemNotes(RoidNotes,(PR.SystemPrefix + PR.LoadedSector)) 
      PR.SaveSectorSave = 0 
   end
end

function PR.LoadSector(SectorId)
   if PR.LoadedSector == SectorId then return end -- already loaded
   if PR.SaveSectorSave ~= 0 then 
   PR.SaveSectorNow()
   end
   
   if PR.LoadedSector ~= SectorId then
      PR.Data = {}
      PR.Data.MaxId = 0
      PR.Data.Usable = 0
      PR.Data.RoidStatus = {}
      PR.Data.RoidList = {}
      PR.Data.Unscanned = 0
      PR.Data.Bots = 0
      
      PR.LoadedSector = SectorId   
      local RoidNotes = LoadSystemNotes(PR.SystemPrefix + PR.LoadedSector )
      local tempD = unspickle(RoidNotes)
      for Cat,Data in pairs (tempD) do
         PR.Data[Cat] = tempD[Cat]           
      end
   end 
   local RoidCounter = 0
   if PR.Data.RoidList ~= nil then               
      for RoidId,Data in pairs (PR.Data.RoidList) do
         RoidCounter = (RoidCounter +1)
      end
   end
   if PR.Data.Usable ~= RoidCounter then PR.Data.Usable = RoidCounter end
   PR.GetCompletionStats()
end

function PR.UpdateRoidStatus(RoidId,Status)
   if Status == nil or RoidId == nil or RoidId == 0 then return end
   if PR.LoadedSector ~= PR.Sector then PR.LoadSector(PR.Sector) end
   if RoidId > PR.Data.MaxId then PR.Data.MaxId = RoidId end 
     
   if Status == "u" or "r" or "j" then
      PR.Data.RoidStatus[RoidId] = Status
      if PR.Data.RoidStatus[RoidId] == "u" then 
         PR.Data.Unscanned = PR.Data.Unscanned -1  
      end
      PR.SaveSector(PR.Sector)
   end
   PR.GetCompletionStats()   
end

function PR.GetRoidStatus(RoidId)
   if RoidId == nil then return end
   return  (PR.Data.RoidStatus[RoidId] or "u")
end
  

function PR.GetCompletionStats()
   if PR.Data.MaxId == nil then return end
   PR.Data.Unscanned = 0
   local rstat = 0
   for Index = 1 , PR.Data.MaxId do
      rstat = PR.GetRoidStatus(Index)
      if rstat == nil or rstat == "u" then PR.Data.Unscanned = PR.Data.Unscanned +1 end
   end
end

function PR.UpdateRoidMass(RoidId,Mass)
   local RoidTable = PR.GetRoid(GetCurrentSectorid(),RoidId)

   if RoidTable["Pt"] == nil then 
      RoidTable["Pt"] = Mass
   else 
      RoidTable["Pt"] = math.floor((RoidTable["Pt"] + RoidTable["Pt"] + Mass) / 3)
   end
   PR.SetRoid(RoidTable)
end




